;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Event routines                                                ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; findTrigger - search the events for a trigger
;   input:  A = trigger id
;           B = # entries in eventTable left to check
;           HL => where in eventTable to start searching
;   output: HL => byte after trigger
;           DE => entry in eventTable
;           B = # entries left in eventTable after this one
;           CA = trigger matched
;------------------------------------------------
findTrigger:
        ld      c,a                             ; C = trigger id
        ld      d,a
        ld      a,b                             ; A = # events to check
        or      a
        ret     z                               ; if none, leave (carry is clear)
        ld      a,d
ftLoop:
        push    hl
#ifdef TI83PLUS
        LDHLIND()                               ; HL = offset to event
        ; check that offset isn't $FFFF
        inc     hl
        ld      a,h
        or      l                               ; did incrementing HL make it $0000?
        dec     hl
#endif
#ifdef TI84PLUSCE
        ld      de,0
        ld      e,(hl)
        inc     hl
        ld      d,(hl)
        push    de
        inc d \ inc e
        ld      a,d
        or      e                               ; was offset $00FFFF?
        pop     hl
#endif
        jr      z,ftEndLoop                     ; if so, not a valid offset entry (event has been disabled)
        ld      de,(pEvents)                    ; DE => start of event data
        add     hl,de                           ; HL => event
        ld      a,(hl)                          ; A = trigger id of this event
        cp      c                               ; does it match?
        jr      z,ftFound
ftEndLoop:
        pop     hl
        inc hl \ inc hl                         ; HL => next eventTable entry
        djnz    ftLoop
        or      a                               ; clear carry
        ret
ftFound:
        pop     de                              ; DE => entry in eventTable
        dec     b                               ; B = # entries 
        inc     hl                              ; HL => byte after trigger
        scf                                     ; set carry to denote trigger matched
        ret

;------------------------------------------------
; trgWalkedOnArea - check events to see if any scripts need to be run
;   input:  DE = x
;           HL = y
;   output: DE = x
;           HL = y
;   notes:  preserves DE HL
;------------------------------------------------
trgWalkedOnArea:
        ld      (tempX),de
        ld      (tempY),hl
        ld      hl,collideObject1
        call    loadPlayerCollideObjectHL       ; put player in collideObject1 (IX is already pointing to player data)
        ld      bc,(numEvents-1)                ; B = # events
        ld      hl,(pEventTable)
twoaFind:
        ld      a,TRG_WALKEDONAREA              ; A = trigger to search for
        call    findTrigger
        jr      nc,twoaEnd
        push    bc
        push    de
        push    hl
        pop     ix                              ; IX => area x,width,y,height
#ifdef TI83PLUS
        ld      l,(ix)
        ld      h,0
        HLTIMES8()
        ld      (collideObject2),hl             ; set x
        ld      a,(ix+1)
        ATIMES8()
        ld      (collideObject2+2),a            ; set width
        ld      l,(ix+2)
        ld      h,0
        HLTIMES8()
        ld      (collideObject2+3),hl           ; set y
        ld      a,(ix+3)
        ATIMES8()
        ld      (collideObject2+5),a            ; set height
#endif
#ifdef TI84PLUSCE
        ld      l,(ix)
        ld      h,8
        mlt     hl
        ld      (collideObject2),hl             ; set x (don't worry about HLU being written, it's about to be overwritten by the width)
        ld      a,(ix+1)
        ATIMES8()
        ld      (collideObject2+2),a            ; set width
        ld      l,(ix+2)
        ld      h,8
        mlt     hl
        ld      (collideObject2+3),hl           ; set y
        ld      a,(ix+3)
        ATIMES8()
        ld      (collideObject2+5),a            ; set height
#endif
        call    checkCollision                  ; check if player is in the area
        pop     hl                              ; HL => eventTable entry
        pop     bc
        jr      nc,twoaFindNext                 ; if player isn't in area, check the rest of eventTable in case there's another event to check
        ; if code reaches here, the script needs to be run
        ex      de,hl                           ; DE => eventTable entry
        push ix \ pop hl
        ld      bc,4
        add     hl,bc                           ; HL => start of script
        call    runScript
twoaEnd:
        ld      de,(tempX)
        ld      hl,(tempY)
        ret
twoaFindNext:
        inc hl \ inc hl
        jr      twoaFind

;------------------------------------------------
; trgBossKilled - check events to see if any scripts need to be run
;   input:  none
;   output: none
;------------------------------------------------
trgBossKilled:
        ld      bc,(numEvents-1)                ; B = # events
        ld      hl,(pEventTable)
        ld      a,TRG_BOSSKILLED                ; A = trigger to search for
        call    findTrigger
        ret     nc
        jp      runScript

;------------------------------------------------
; trgCountdownStarted - check events to see if any scripts need to be run
;   input:  none
;   output: none
;------------------------------------------------
trgCountdownStarted:
        ld      bc,(numEvents-1)
        ld      hl,(pEventTable)
        ld      a,TRG_COUNTDOWNSTARTED
        call    findTrigger
        ret     nc
        jp      runScript

;------------------------------------------------
; trgFireDoor - check events to see if any scripts need to be run
;   input:  none
;   output: none
;------------------------------------------------
trgFireDoor:
        ld      bc,(numEvents-1)
        ld      hl,(pEventTable)
        ld      a,TRG_FDOORCLOSED
        call    findTrigger
        ret     nc
        jp      runScript

;------------------------------------------------
; trgPowerDome - check events to see if any scripts need to be run
;   input:  none
;   output: none
;------------------------------------------------
trgPowerDome:
        ld      bc,(numEvents-1)
        ld      hl,(pEventTable)
        ld      a,TRG_PDOMEDESTROYED
        call    findTrigger
        ret     nc
        jp      runScript

;------------------------------------------------
; trgHitWall - check events to see if any scripts need to be run
;   input:  none
;   output: none
;------------------------------------------------
trgHitWall:
        ld      bc,(numEvents-1)
        ld      hl,(pEventTable)
        ld      a,TRG_HITWALL
        call    findTrigger
        ret     nc
        jr      runScript

;------------------------------------------------
; trgCounterFinished - check events to see if any scripts need to be run
;   input:  A = counter #
;   output: none
;------------------------------------------------
trgCounterFinished:
        ld      (tempB),a
        ld      bc,(numEvents-1)
        ld      hl,(pEventTable)
tcfFind:
        ld      a,TRG_COUNTERFINISHED
        call    findTrigger
        ret     nc
        push    bc
        push    de
        push    hl
        ld      a,(tempB)
        cp      (hl)                            ; does counter # match?
        pop     de
        pop     hl                              ; HL => eventTable entry
        pop     bc
        jr      nz,tcfFindNext                  ; if not, check the rest of eventTable in case there's another event to check
        ; if code reaches here, the script needs to be run
        ex      de,hl                           ; DE => eventTable entry
        inc     hl                              ; HL => start of script
        jr      runScript
tcfFindNext:
        inc hl \ inc hl
        jr      tcfFind

;------------------------------------------------
; trgLevelStarted - check events to see if any scripts need to be run
;   input:  none
;   output: none
;------------------------------------------------
trgLevelStarted:
        ld      bc,(numEvents-1)
        ld      hl,(pEventTable)
        ld      a,TRG_LEVELSTARTED
        call    findTrigger
        ret     nc
        jr      runScript

;------------------------------------------------
; trgLifeBomb - check events to see if any scripts need to be run
;   input:  none
;   output: none
;------------------------------------------------
trgLifeBomb:
        ld      bc,(numEvents-1)
        ld      hl,(pEventTable)
        ld      a,TRG_LIFEBOMB
        call    findTrigger
        ret     nc
        jr      runScript

;------------------------------------------------
; trgFrame4 - check events to see if any scripts need to be run
;   input:  none
;   output: none
;------------------------------------------------
trgFrame4:
        ld      bc,(numEvents-1)
        ld      hl,(pEventTable)
        ld      a,TRG_FRAME4
        call    findTrigger
        ret     nc
        jr      runScript

;------------------------------------------------
; trgIntex - check events to see if any scripts need to be run
;   input:  none
;   output: none
;------------------------------------------------
trgIntex:
        ld      bc,(numEvents-1)
        ld      hl,(pEventTable)
        ld      a,TRG_INTEX
        call    findTrigger
        ret     nc
        jr      runScript

;------------------------------------------------
; runScript - run an event script
;   input:  HL => start of script
;           DE => entry in eventTable
;   output: none
;------------------------------------------------
runScript:
        ld      a,(hl)                          ; A = command
        inc     a                               ; $FF?
        ret     z
        dec     a                               ; $00
        jr      z,cmdClearEvent
        dec     a                               ; $01
        jr      z,cmdUpdateTile
        dec     a                               ; $02
        jr      z,cmdIniBoss
        dec     a                               ; $03
        jr      z,cmdStartCountdown
        dec     a                               ; $04
        jr      z,cmdIncCounter
        dec     a                               ; $05
        jp      z,cmdShowText
        dec     a                               ; $06
        jp      z,cmdTeleport
        dec     a                               ; $07
        jp      z,cmdChangeEnemy
        dec     a                               ; $08
        jp      z,cmdObjComplete
        dec     a                               ; $09
        jp      z,cmdEp3EasterEgg
        ; if execution reached here, the command wasn't recognised
        ; leave the script to avoid any potential crashes from invalid script data
        ret
runScriptNext:
        inc     hl                              ; HL => next command
        jr      runScript

;------------------------------------------------
; cmdClearEvent - clear the event offset entry in eventTable so that event doesn't get run again
;------------------------------------------------
cmdClearEvent:
        ld      a,$FF
        ld      (de),a
        inc     de
        ld      (de),a
        dec     de                              ; return DE => start of eventTable entry, in case this routine ends up getting called again
        jr      runScriptNext

;------------------------------------------------
; cmdUpdateTile - change a tile
;------------------------------------------------
cmdUpdateTile:
        push    de
        ; load the 3 bytes of data (2 bytes = offset, 1 byte = new tile id)
        inc     hl
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      e,(hl)
        inc     hl
        ld      d,(hl)                          ; DE = offset
        inc     hl
        ld      a,(hl)                          ; A = tile id
        push    hl
        ld      hl,(tileDataPtr)
        add     hl,de                           ; HL => tile in level data
        call    putTile
        pop     hl
        pop     de
        jr      cmdClearEvent

;------------------------------------------------
; cmdIniBoss - start a boss fight
;------------------------------------------------
cmdIniBoss:
        inc     hl                              ; HL => boss data
        push    de
        push    hl
        call    iniBoss
        pop     hl
        inc hl \ inc hl                         ; HL => last byte of boss data (will be inc'd to next command later)
        pop     de
        jr      cmdClearEvent

;------------------------------------------------
; cmdStartCountdown - try to start countdown sequence (will fail if already running)
;------------------------------------------------
cmdStartCountdown:
        inc     hl                              ; HL => time value to use
        ld      ix,levelFlags
        bit     LFLAG_COUNTDOWNON,(ix)
        jr      nz,afterStartCountdown          ; if it's already running, do nothing
        set     LFLAG_COUNTDOWNON,(ix)
        ld      a,(hl)
        ld      (time),a
        push    hl
        push    de
        call    trgCountdownStarted
#ifdef TI84PLUSCE
        call    iniCountdownPalette
#endif
        pop     de
        pop     hl
afterStartCountdown:
        jr      runScriptNext

;------------------------------------------------
; cmdIncCounter - increment a counter
;------------------------------------------------
cmdIncCounter:
        inc     hl                              ; HL => counter id#
#ifdef TI83PLUS
        ld b,0 \ ld c,(hl)                      ; BC = counter id#
#endif
#ifdef TI84PLUSCE
        ld bc,0 \ ld c,(hl)
#endif
        ld ix,counters \ add ix,bc              ; IX => counter
        inc     hl                              ; HL => counter target value
        ld      a,(ix)                          ; A = counter
        cp      (hl)
        jr      nc,runScriptNext                ; if counter has already reached target, nothing to do
        inc     a
        ld      (ix),a                          ; save updated counter
        cp      (hl)                            ; check if it reached targer after the increment
        jr      c,runScriptNext                 ; if not, continue with the script
        push    hl
        push    de
        ld      a,c                             ; A = counter id#
        call    trgCounterFinished
        pop     de
        pop     hl
        jp      runScriptNext

;------------------------------------------------
; cmdShowText - show event text
;------------------------------------------------
cmdShowText:
        ld      a,$FF
        ld (de),a \ inc de \ ld (de),a          ; prevent event from occurring again
        inc     hl                              ; HL => text data
#ifdef TI83PLUS
        push    hl
        call    saveVideo
        pop     hl
        bcall(ab5_showEventText)
        jp      restoreVideo
#endif
#ifdef TI84PLUSCE
        jp      showEventText
#endif

;------------------------------------------------
; cmdTeleport - teleport the player
;------------------------------------------------
cmdTeleport:
        inc     hl                              ; HL => teleport coords
        ld      (pTeleport),hl
        ld      a,(miscFlags)
        set     teleportCmd,a
        ld      (miscFlags),a
        inc     hl
        jp      runScriptNext

;------------------------------------------------
; cmdChangeEnemy - change an enemy type
;------------------------------------------------
cmdChangeEnemy:
        push    de
        inc     hl
        ld      ix,enemy1
        bit     0,(hl)
        jr      z,doChangeEnemy
        inc     ix
doChangeEnemy:
        ld      a,(ix)                          ; A = current enemy type/level in this slot
        and     %00011111
        inc     a
        ld      c,a                             ; C = current enemy type for enemyTable checks
        inc     hl
        push    hl
        ld      l,(hl)                          ; L = new enemy type
        ld      a,(ix)
        and     %11100000                       ; mask out type but save level
        or      l
        ld      (ix),a                          ; save new enemy type/level
        ld      ix,enemyTable
        ld      b,NUM_ENEMIES
        inc     l                               ; L = new enemy type for enemyTable update
        ld      de,ENEMY_SIZE
changeEnemiesLoop:
        ld      a,(ix)
        cp      c
        jr      nz,endChangeEnemiesLoop
        ld      (ix),l
endChangeEnemiesLoop:
        add     ix,de
        djnz    changeEnemiesLoop
        pop     hl
        pop     de
        jp      runScriptNext

;------------------------------------------------
; cmdObjComplete
;------------------------------------------------
cmdObjComplete:
        ld      ix,levelFlags
        set     LFLAG_CANFINISH,(ix)
        jp      cmdClearEvent

;------------------------------------------------
; cmdEp3EasterEgg
;------------------------------------------------
cmdEp3EasterEgg:
        ; clear trigger
        ld      a,$FF
        ld      (de),a
        inc     de
        ld      (de),a
#ifdef TI83PLUS
        call    saveVideo
        bcall(ab5_ep3EasterEgg)
#endif
#ifdef TI84PLUSCE
        ; show the text pages
        ld      hl,e3egg
        ld      b,(hl)                          ; B = # of pages
        inc     hl                              ; HL => start of text data
e3EasterEggLoop:
        push    bc
        push    hl
        call    modifyText
        call    fadeOutClear
        pop     hl
        call    displayPage
        push    hl
        call    wait2ndEnter
        pop     hl
        pop     bc
        djnz    e3EasterEggLoop
#endif
        ld      bc,ACH_EP3EASTER_BIT*256+ACH_EP3EASTER_ID
        ld      de,ACH_EP3EASTER_OFFSET
        ld      hl,viewNewAchievement
        bcall(ab5_setAchievement)
#ifdef TI83PLUS
        jp      restoreVideo
#endif
#ifdef TI84PLUSCE
        jp      fadeToGame
#endif

.end
