;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Save/load state routines                                      ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; saveState - attempt to save the game state and quit
;   input:  none
;   output: none
;   notes:  will simply ret to main game loop if not enough free memory
;------------------------------------------------
saveState:
        ; check enough free RAM and create appvar to store vars
        ld      hl,VARS_MEM_NEEDED              ; HL = required memory
        bcall(_enoughMem)                       ; check free memory
        ret     c                               ; if not enough, return to game loop
#ifdef TI83PLUS
        pop hl \ pop hl \ pop hl                ; pop the bcalls
        pop hl \ pop hl \ pop hl
        ld      a,$40
        call    $000B
        out     ($10),a                         ; ensure Z-offset is reset
#endif
#ifdef TI84PLUSCE
        pop     hl                              ; pop the calls
        pop     hl
#endif
        
        ; if a co-op game and player 1 is no longer active, then make p1 position equal to p2 (but slightly to the left)
        ; this is because when loading the state, p2 is always moved to p1 location
        ld      a,(coop)
        or      a
        jr      z,ssAfterPMove
        ld      a,(p1p)
        or      a
        jr      nz,ssAfterPMove
        ld      hl,(p2x)
        ld      de,-6
        add     hl,de
        ld      (p1x),hl
        ld      hl,(p2y)
        ld      (p1y),hl

ssAfterPMove:

        call    checkVarsFile
        ld      hl,VARS_SIZE+2                  ; enough space for all variables plus a checksum
        bcall(_createappvar)                    ; create vars appvar
        inc de \ inc de                         ; DE => start of storage
        push    de
        ld      bc,VARS_SIZE
        push    bc
        ld      hl,vars
        ldir                                    ; copy variables to file
        pop     bc
        pop     hl                              ; HL => start of variables file data
        ld      ix,0
#ifdef TI83PLUS
        ld      d,0
#endif
#ifdef TI84PLUSCE
        ld      de,0
#endif
ssVarsChecksumLoop:
        ld      e,(hl)                          ; DE = byte value (D is always 0)
        add     ix,de                           ; add it to total
        inc     hl
        dec     bc
        ld      a,b
        or      c
        jr      nz,ssVarsChecksumLoop
        push    hl                              ; save pointer to where checksum will be saved (at end of vars file)
        ld      hl,(tempFile)
        ld      bc,LEVEL_DATA_SIZE
ssLvlChecksumLoop:
        ld      e,(hl)
        add     ix,de
        inc     hl
        dec     bc
        ld      a,b
        or      c
        jr      nz,ssLvlChecksumLoop
        pop     hl                              ; HL => where to save checksum
        push ix \ pop de                        ; DE = checksum total
        ld      (hl),e
        inc     hl
        ld      (hl),d                          ; save checksum to end of vars file
        
        ld      hl,tempAppvar
        MOVE9()
        bcall(_chkfindsym)
        bcallnc(_arc_unarc)                     ; archive the level data file
        
        ld      hl,varsAppvar
        MOVE9()
        bcall(_chkfindsym)
        bcallnc(_arc_unarc)                     ; archive the vars file

        jp      appQuit

;------------------------------------------------
; loadState - attempt to load a saved game state
;   input:  BDE => memory location
;           HL => VAT entry
;           (OP1) = vars file name
;   output: none
;------------------------------------------------
loadState:
#ifdef TI83PLUS
        ld      a,b
        or      a                               ; is vars file archived?
        bcallnz(_arc_unarc)                     ; if so, unarchive it
        ex      de,hl
        inc hl \ inc hl                         ; HL => start of vars file data
#endif
#ifdef TI84PLUSCE
        ld      a,COLOUR_WHITE
        ld      (__psColour),a
        call    getDataPtr
        ex      de,hl
#endif
        ; load variables from vars file
        ld      de,vars
        ld      bc,VARS_SIZE
        ldir
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      e,(hl)
        inc     hl
        ld      d,(hl)                          ; DE = checksum that needs to be matched
        push    de                              ; save it for now
        ld      hl,vars
#ifdef TI83PLUS
        ld      d,0
#endif
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      ix,0
        ld      bc,VARS_SIZE
lsVarsChecksumLoop:
        ld      e,(hl)
        add     ix,de
        inc     hl
        dec     bc
        ld      a,b
        or      c
        jr      nz,lsVarsChecksumLoop
        push    ix                              ; save checksum running total
        
        call    checkVarsFile                   ; delete the vars file

        ; locate temp level file and unarchive it if required
lsFindLevelAppvar:
        ld      hl,tempAppvar
        MOVE9()
        bcall(_chkfindsym)
        pop     ix
        pop     hl
        ret     c                               ; if not found, can't load state, so ret back to standard game loading routine
        push    hl
        push    ix
#ifdef TI83PLUS
        ld      a,b
        or      a
#endif
#ifdef TI84PLUSCE
        call    _chkInRam
#endif
        jr      z,lsLevelAppvarInRam
        bcall(_arc_unarc)
        jr      lsFindLevelAppvar
lsLevelAppvarInRam:
        inc de \ inc de                         ; DE => start of temp file data
        ld      (tempFile),de                   ; save pointer to unarchived temp file
#ifdef TI83PLUS
        ld      hl,LEVEL_DATA_SIZE
        add     hl,de
        ld      (pTiles),hl
#endif
        pop     ix                              ; IX = current checksum running total
        ex      de,hl                           ; HL => start of temp file data
#ifdef TI83PLUS
        ld      d,0
#endif
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      bc,LEVEL_DATA_SIZE
lsLvlChecksumLoop:
        ld      e,(hl)
        add     ix,de
        inc     hl
        dec     bc
        ld      a,b
        or      c
        jr      nz,lsLvlChecksumLoop
        push ix \ pop de                        ; DE = checksum total
#ifdef TI84PLUSCE
        call    _setDEUTo0
#endif
        pop     hl                              ; HL = value to check against
        bcall(_cphlde)
        ret     nz                              ; if not matching, can't load state, so ret back to standard game loading routine
        
        ; if code reached here, then the load state was successful
        pop     hl                              ; pop the call from the game loading routine
        call    loadAppvar                      ; load the appvar with high scores, etc.
        jp      nz,quit                         ; if that failed, can't run the game
        ld      hl,miscFlags
        set     loadingState,(hl)               ; set loading state flag

        ; reset the tileDataPtr
        ld      hl,(tempFile)
        ; first skip through the mission text (which is $FF terminated)
lsFindTiles:
        ld      a,(hl)
        inc     hl
        inc     a
        jr      nz,lsFindTiles
        ld      de,LEVEL_HEADER
        add     hl,de                           ; HL => tile data
        ld      (tileDataPtr),hl
        ; get pointer to start of events
        push    hl
        ld      hl,(levelWidth)                 ; L = width, H = height
        bcall(_htimesl)
        pop     de
        add     hl,de                           ; HL => start of event data
        ld      a,(hl)
        or      a                               ; are there any events?
        call    nz,iniEvents

#ifdef TI83PLUS
        bcall(ab5_drawLevel)                    ; draw level and resume game
#endif
#ifdef TI84PLUSCE
        call    loadDefaultPalette
        call    loadLevelGfx
        call    loadEnemySprites
        ld      a,(state)
        or      a
        call    nz,loadBossSprites
        call    loadHud
        ld      hl,levelFlags
        bit     LFLAG_COUNTDOWNON,(hl)
        call    nz,fullCountdownPalette
        jp      drawLevel
#endif

;------------------------------------------------
; checkVarsFile - check for temp vars file and delete it if it exists
;   input:  none
;   output: (OP1) = vars file name
;------------------------------------------------
checkVarsFile:
        ld      hl,varsAppvar                   ; HL => vars appvar file name
        MOVE9()                                 ; move it to OP1
        bcall(_chkfindsym)                      ; check if it exists
        bcallnc(_delvararc)                     ; if so, delete it
        ret

.end
